import * as i0 from '@angular/core';
import { InjectionToken, input, inject, booleanAttribute, numberAttribute, EventEmitter, signal, Renderer2, effect, Component, ChangeDetectionStrategy, Output, ViewChild, makeEnvironmentProviders } from '@angular/core';
import { generateQrCodeSVG$, generateQrCodeImage$, generateQrCodeCanvas$ } from 'dfts-qrcode';

function downloadQRCode(dataURL, type, filename = 'qrcode') {
    // split into two parts
    const parts = dataURL.split(';base64,');
    // hold the content type
    const imageType = parts[0].split(':')[1];
    // decode base64 string
    const decodedData = atob(parts[1]);
    // create unit8array of size same as row data length
    const uInt8Array = new Uint8Array(decodedData.length);
    // insert all character code into uint8array
    for (let i = 0; i < decodedData.length; ++i) {
        uInt8Array[i] = decodedData.charCodeAt(i);
    }
    // return blob image after conversion
    const blobData = new Blob([uInt8Array], { type: imageType });
    // saves as image
    const blob = new Blob([blobData], { type: `image/${type === 'png' ? 'png' : 'svg+xml'}` });
    const url = URL.createObjectURL(blob);
    const link = document.createElement('a');
    link.href = url;
    link.download = filename;
    link.click();
}

const QRCODE_ALLOW_EMPTY_STRING = new InjectionToken('QRCODE_ALLOW_EMPTY_STRING', {
    factory: () => false,
});
const QRCODE_COLOR_DARK = new InjectionToken('QRCODE_COLOR_DARK', {
    factory: () => '#000000',
});
const QRCODE_COLOR_LIGHT = new InjectionToken('QRCODE_COLOR_LIGHT', {
    factory: () => '#ffffff',
});
const QRCODE_CSS_CLASS = new InjectionToken('QRCODE_CSS_CLASS', {
    factory: () => 'qrcode',
});
const QRCODE_ELEMENT_TYPE = new InjectionToken('QRCODE_ELEMENT_TYPE', {
    factory: () => 'img',
});
const QRCODE_ERROR_CORRECTION_LEVEL = new InjectionToken('QRCODE_ERROR_CORRECTION_LEVEL', {
    factory: () => 'L',
});
const QRCODE_MARGIN = new InjectionToken('QRCODE_MARGIN', {
    factory: () => 4,
});
const QRCODE_IMAGE_SRC = new InjectionToken('QRCODE_IMAGE_SRC', {
    factory: () => undefined,
});
const QRCODE_IMAGE_HEIGHT = new InjectionToken('QRCODE_IMAGE_HEIGHT', {
    factory: () => 40,
});
const QRCODE_IMAGE_WIDTH = new InjectionToken('QRCODE_IMAGE_WIDTH', {
    factory: () => 40,
});
const QRCODE_VERSION = new InjectionToken('QRCODE_VERSION', {
    factory: () => -1,
});
const QRCODE_SIZE = new InjectionToken('QRCODE_WIDTH', {
    factory: () => 5,
});

class QRCodeComponent {
    #viewChild;
    set qrcElement(it) {
        this.#viewChild.set(it);
    }
    #renderer;
    constructor() {
        this.allowEmptyString = input(inject(QRCODE_ALLOW_EMPTY_STRING), { transform: booleanAttribute });
        this.colorDark = input(inject(QRCODE_COLOR_DARK));
        this.colorLight = input(inject(QRCODE_COLOR_LIGHT));
        this.cssClass = input(inject(QRCODE_CSS_CLASS));
        this.elementType = input(inject(QRCODE_ELEMENT_TYPE));
        this.errorCorrectionLevel = input(inject(QRCODE_ERROR_CORRECTION_LEVEL));
        this.version = input(inject(QRCODE_VERSION), { transform: numberAttribute });
        this.size = input(inject(QRCODE_SIZE), { transform: numberAttribute });
        this.margin = input(inject(QRCODE_MARGIN), { transform: numberAttribute });
        this.imageSrc = input(inject(QRCODE_IMAGE_SRC));
        this.imageHeight = input(inject(QRCODE_IMAGE_HEIGHT), { transform: numberAttribute });
        this.imageWidth = input(inject(QRCODE_IMAGE_WIDTH), { transform: numberAttribute });
        this.alt = input();
        this.ariaLabel = input();
        this.title = input();
        this.data = input('');
        this.qrCodeDataUrl = new EventEmitter();
        this.#viewChild = signal(undefined);
        this.#renderer = inject(Renderer2);
        effect(() => {
            const viewChild = this.#viewChild();
            if (!viewChild) {
                return;
            }
            let data = this.data();
            const validQrData = this.#isValidQrCodeText(data);
            if (!this.allowEmptyString() && !validQrData) {
                console.error('[dfx-qrcode] Field `data` is empty, set \'allowEmptyString="true"\' to overwrite this behaviour.');
                return;
            }
            else if (this.allowEmptyString() && !validQrData) {
                data = ' ';
            }
            try {
                const config = {
                    errorCorrectionLevel: this.errorCorrectionLevel(),
                    version: this.version(),
                    size: this.size(),
                    alt: this.alt(),
                    title: this.title(),
                    ariaLabel: this.ariaLabel(),
                    margin: this.margin(),
                    colors: {
                        colorLight: this.colorLight(),
                        colorDark: this.colorDark(),
                    },
                    image: {
                        src: this.imageSrc(),
                        width: this.imageWidth(),
                        height: this.imageHeight(),
                    },
                };
                switch (this.elementType()) {
                    case 'canvas': {
                        generateQrCodeCanvas$(data, config, this.#renderer.createElement('canvas')).then((element) => {
                            this.#renderElement(viewChild, element);
                            this.qrCodeDataUrl.emit(element.toDataURL());
                        });
                        break;
                    }
                    case 'img': {
                        generateQrCodeImage$(data, config, this.#renderer.createElement('canvas'), this.#renderer.createElement('img')).then(({ image, dataUrl }) => {
                            this.#renderElement(viewChild, image);
                            this.qrCodeDataUrl.emit(dataUrl);
                        });
                        break;
                    }
                    case 'svg': {
                        generateQrCodeSVG$(data, config).then(({ svg, dataUrl }) => {
                            this.#renderElement(viewChild, svg);
                            this.qrCodeDataUrl.emit(dataUrl);
                        });
                        break;
                    }
                    default:
                        console.error(`[dfx-qrcode] Error: Unknown elementType "${this.elementType()}"`);
                }
            }
            catch (e) {
                console.error('[dfx-qrcode] Error generating QR Code:', e);
            }
        });
    }
    #isValidQrCodeText(data) {
        return data !== null && data !== undefined && data.length > 0;
    }
    #renderElement(viewChild, element) {
        for (const node of viewChild.nativeElement.childNodes) {
            this.#renderer.removeChild(viewChild.nativeElement, node);
        }
        this.#renderer.appendChild(viewChild.nativeElement, element);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.3.0", ngImport: i0, type: QRCodeComponent, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.1.0", version: "17.3.0", type: QRCodeComponent, isStandalone: true, selector: "qrcode", inputs: { allowEmptyString: { classPropertyName: "allowEmptyString", publicName: "allowEmptyString", isSignal: true, isRequired: false, transformFunction: null }, colorDark: { classPropertyName: "colorDark", publicName: "colorDark", isSignal: true, isRequired: false, transformFunction: null }, colorLight: { classPropertyName: "colorLight", publicName: "colorLight", isSignal: true, isRequired: false, transformFunction: null }, cssClass: { classPropertyName: "cssClass", publicName: "cssClass", isSignal: true, isRequired: false, transformFunction: null }, elementType: { classPropertyName: "elementType", publicName: "elementType", isSignal: true, isRequired: false, transformFunction: null }, errorCorrectionLevel: { classPropertyName: "errorCorrectionLevel", publicName: "errorCorrectionLevel", isSignal: true, isRequired: false, transformFunction: null }, version: { classPropertyName: "version", publicName: "version", isSignal: true, isRequired: false, transformFunction: null }, size: { classPropertyName: "size", publicName: "size", isSignal: true, isRequired: false, transformFunction: null }, margin: { classPropertyName: "margin", publicName: "margin", isSignal: true, isRequired: false, transformFunction: null }, imageSrc: { classPropertyName: "imageSrc", publicName: "imageSrc", isSignal: true, isRequired: false, transformFunction: null }, imageHeight: { classPropertyName: "imageHeight", publicName: "imageHeight", isSignal: true, isRequired: false, transformFunction: null }, imageWidth: { classPropertyName: "imageWidth", publicName: "imageWidth", isSignal: true, isRequired: false, transformFunction: null }, alt: { classPropertyName: "alt", publicName: "alt", isSignal: true, isRequired: false, transformFunction: null }, ariaLabel: { classPropertyName: "ariaLabel", publicName: "ariaLabel", isSignal: true, isRequired: false, transformFunction: null }, title: { classPropertyName: "title", publicName: "title", isSignal: true, isRequired: false, transformFunction: null }, data: { classPropertyName: "data", publicName: "data", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { qrCodeDataUrl: "qrCodeDataUrl" }, viewQueries: [{ propertyName: "qrcElement", first: true, predicate: ["qrcElement"], descendants: true, static: true }], ngImport: i0, template: `<div #qrcElement [class]="cssClass()"></div>`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.3.0", ngImport: i0, type: QRCodeComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'qrcode',
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    standalone: true,
                    template: `<div #qrcElement [class]="cssClass()"></div>`,
                }]
        }], ctorParameters: () => [], propDecorators: { qrCodeDataUrl: [{
                type: Output
            }], qrcElement: [{
                type: ViewChild,
                args: ['qrcElement', { static: true }]
            }] } });

var QRCodeFeatureKind;
(function (QRCodeFeatureKind) {
    QRCodeFeatureKind[QRCodeFeatureKind["ALLOW_EMPTY_STRING"] = 0] = "ALLOW_EMPTY_STRING";
    QRCodeFeatureKind[QRCodeFeatureKind["COLOR_DARK"] = 1] = "COLOR_DARK";
    QRCodeFeatureKind[QRCodeFeatureKind["COLOR_LIGHT"] = 2] = "COLOR_LIGHT";
    QRCodeFeatureKind[QRCodeFeatureKind["CSS_CLASS"] = 3] = "CSS_CLASS";
    QRCodeFeatureKind[QRCodeFeatureKind["ELEMENT_TYPE"] = 4] = "ELEMENT_TYPE";
    QRCodeFeatureKind[QRCodeFeatureKind["ERROR_CORRECTION_LEVEL"] = 5] = "ERROR_CORRECTION_LEVEL";
    QRCodeFeatureKind[QRCodeFeatureKind["IMAGE"] = 6] = "IMAGE";
    QRCodeFeatureKind[QRCodeFeatureKind["MARGIN"] = 7] = "MARGIN";
    QRCodeFeatureKind[QRCodeFeatureKind["VERSION"] = 8] = "VERSION";
    QRCodeFeatureKind[QRCodeFeatureKind["SIZE"] = 9] = "SIZE";
})(QRCodeFeatureKind || (QRCodeFeatureKind = {}));

var QRCodeImageFeatureKind;
(function (QRCodeImageFeatureKind) {
    QRCodeImageFeatureKind[QRCodeImageFeatureKind["SRC"] = 0] = "SRC";
    QRCodeImageFeatureKind[QRCodeImageFeatureKind["WIDTH"] = 1] = "WIDTH";
    QRCodeImageFeatureKind[QRCodeImageFeatureKind["HEIGHT"] = 2] = "HEIGHT";
})(QRCodeImageFeatureKind || (QRCodeImageFeatureKind = {}));

function provideQRCode(...features) {
    return makeEnvironmentProviders([features.map((it) => it.providers)]);
}
function withAllowEmptyString(allowEmptyString) {
    return {
        kind: QRCodeFeatureKind.ALLOW_EMPTY_STRING,
        providers: [{ provide: QRCODE_ALLOW_EMPTY_STRING, useValue: allowEmptyString }],
    };
}
function withColorDark(colorDark) {
    return {
        kind: QRCodeFeatureKind.COLOR_DARK,
        providers: [{ provide: QRCODE_COLOR_DARK, useValue: colorDark }],
    };
}
function withColorLight(colorLight) {
    return {
        kind: QRCodeFeatureKind.COLOR_LIGHT,
        providers: [{ provide: QRCODE_COLOR_LIGHT, useValue: colorLight }],
    };
}
function withCssClass(cssClass) {
    return {
        kind: QRCodeFeatureKind.CSS_CLASS,
        providers: [{ provide: QRCODE_CSS_CLASS, useValue: cssClass }],
    };
}
function withElementType(elementType) {
    return {
        kind: QRCodeFeatureKind.ELEMENT_TYPE,
        providers: [{ provide: QRCODE_ELEMENT_TYPE, useValue: elementType }],
    };
}
function withErrorCorrectionLevel(errorCorrectionLevel) {
    return {
        kind: QRCodeFeatureKind.ERROR_CORRECTION_LEVEL,
        providers: [
            {
                provide: QRCODE_ERROR_CORRECTION_LEVEL,
                useValue: errorCorrectionLevel,
            },
        ],
    };
}
function withMargin(margin) {
    return {
        kind: QRCodeFeatureKind.MARGIN,
        providers: [{ provide: QRCODE_MARGIN, useValue: margin }],
    };
}
function withImage(...features) {
    return {
        kind: QRCodeFeatureKind.IMAGE,
        providers: features.map((it) => it.provider),
    };
}
function withImageSrc(src) {
    return {
        kind: QRCodeImageFeatureKind.SRC,
        provider: { provide: QRCODE_IMAGE_SRC, useValue: src },
    };
}
function withImageWidth(width) {
    return {
        kind: QRCodeImageFeatureKind.WIDTH,
        provider: { provide: QRCODE_IMAGE_SRC, useValue: width },
    };
}
function withImageHeight(height) {
    return {
        kind: QRCodeImageFeatureKind.HEIGHT,
        provider: { provide: QRCODE_IMAGE_SRC, useValue: height },
    };
}
function withVersion(version) {
    return {
        kind: QRCodeFeatureKind.VERSION,
        providers: [{ provide: QRCODE_VERSION, useValue: version }],
    };
}
function withSize(size) {
    return {
        kind: QRCodeFeatureKind.SIZE,
        providers: [{ provide: QRCODE_SIZE, useValue: size }],
    };
}

/**
 * Generated bundle index. Do not edit.
 */

export { QRCODE_ALLOW_EMPTY_STRING, QRCODE_COLOR_DARK, QRCODE_COLOR_LIGHT, QRCODE_CSS_CLASS, QRCODE_ELEMENT_TYPE, QRCODE_ERROR_CORRECTION_LEVEL, QRCODE_IMAGE_HEIGHT, QRCODE_IMAGE_SRC, QRCODE_IMAGE_WIDTH, QRCODE_MARGIN, QRCODE_SIZE, QRCODE_VERSION, QRCodeComponent, QRCodeFeatureKind, QRCodeImageFeatureKind, downloadQRCode, provideQRCode, withAllowEmptyString, withColorDark, withColorLight, withCssClass, withElementType, withErrorCorrectionLevel, withImage, withImageHeight, withImageSrc, withImageWidth, withMargin, withSize, withVersion };
//# sourceMappingURL=dfx-qrcode.mjs.map
